import os
import re
import numpy as np
from collections import Counter

def transform_JASPAR_to_pfm(path_to_jaspar: str, out_path: str):
    with open(path_to_jaspar, "r") as jaspar:
        with open(out_path, "w") as out:
            for line in jaspar:
                if line.startswith(">"):
                    out.write(line)
                    counter = 0
                elif line.startswith(("A", "C", "G", "T")):
                    line = re.sub('A|G|C|T|\[|\]|\n', '', line).strip()
                    line = " ".join(line.split()) + "\n"
                    out.write(line)
                    counter += 1
                if counter == 4:
                    out.write("\n")
                    counter = 0




def read_pwms_from_pmf(path_to_pfm: str, pseudocount: float = 1.0) -> dict:
    """reads position count matrices from pmf file and transforms them to position wheight matrices

    Args:
        path_to_pfm (str): path to pmf file that should be read in
        pseudocount (float, optional): pseudocount, to prevent a devision by zero later. Defaults to 1.0.

    Returns:
        dict: dictionary with pwms and Metainformation. the rows represent A,C,G,T
    """


    pwms: dict = {}
    assert path_to_pfm[-4:] == ".pfm"
    assert os.path.exists(path_to_pfm)

    with open(path_to_pfm, 'r') as fp:
        line_counter = 0
        for line in fp:
            if line[0] == '>':  # header lines (with name and metainfo)
                name = line[1:].split()[0].strip()
                tf = line.split()[1].strip()
                pwms[name] = {'tf': tf, 'pwm': ''}
                line_counter = 1

            elif line_counter > 0:
                line_counter += 1
                pwms[name]['pwm'] = pwms[name]['pwm'] + line.replace('/n', ' ')

                if line_counter == 5:  # fourth line of array, but we added 1 before, so 5
                    if "." in pwms[name]['pwm']:  # Float values
                        new_pwm = np.fromstring(pwms[name]['pwm'], dtype=float, sep=' ').reshape((4, -1))
                    else: # int values
                        new_pwm = np.fromstring(pwms[name]['pwm'], dtype=int, sep=' ').reshape((4, -1))
                    new_pwm = new_pwm + pseudocount  # to prevent division by zero problems later (this is a standard procedure)
                    new_pwm = new_pwm / new_pwm.sum(axis=0)
                    pwms[name]['pwm'] = new_pwm
                    line_counter = 0

            else:  # empty line
                pass

    return pwms


def calculate_background(path: str = False) -> np.ndarray:
    """calculates background probabilities if given path. Defaults to the backgrounds probabilities of the human genome if no path given.

    Args:
        path (str, optional): Path to fasta file with dna sequence from wich to calculate background probability. Defaults to the backgrounds probabilities of the human genome if no path given
    Returns:
        background_probabilities (np.ndarray):  Array with background probabilities (order: A,C,G,T,N)
    """

    if not path:  # assume it is a human genome
        return np.array([0.28079571, 0.19395748, 0.19479377, 0.28169493, 0.0487581])  # ACGTN
    letter_counter = Counter()
    with open(path, 'r') as fp:
        for line in fp:
            if line[0] == ">":
                if "Primary Assembly" in line:
                    reading = True
                    print(line)
                    print()
                else:
                    reading = False
            elif reading:
                letter_counter.update(Counter(line.upper()))
            else:
                pass
    background = np.array(
        [letter_counter["A"], letter_counter["C"], letter_counter["G"], letter_counter["T"], letter_counter["N"]],
        dtype=np.float64)
    background = background / background.sum()
    return background

def sequence2nparray(sequence: str) -> np.ndarray:
    array = sequence.upper().replace("A", "0").replace("C", "1").replace("G", '2').replace("T", '3').replace("N", '4')
    array = np.array(list(array), dtype=np.uint8)
    return array

def extractSequences(path:str) -> dict:

    with open(path, 'r') as fp:
        chromosomes = fp.readlines()
        chromosomes = ''.join(chromosomes)
    chromosomes = chromosomes.split('>')

    sequences = {}
    just_primary_assembly = True
    for chr in chromosomes:
        if not '\n' in chr:
            continue
        chr = chr.split('\n', maxsplit=1)
        header = chr[0]
        chr = chr[1]
        if "chromosome " in header:
                if just_primary_assembly and "Primary Assembly" in header and not "unlocalized" in header:
                    sequences[header] = chr
                    sequences[header] = sequences[header].replace('\n','').upper()
                    sequences[header] = re.sub(r'[^AGCTN]', 'N', sequences[header])
                    sequences[header] = sequence2nparray(sequences[header])
                    print(header)

    return sequences
